@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 OpenedHand Ltd.
+ * Copyright (C) 2007, 2008 OpenedHand Ltd.
  *
  * Authors: Jorn Baayen <jorn@openedhand.com>
  *
@@ -39,7 +39,7 @@ G_DEFINE_TYPE (GUPnPDIDLLiteWriter,
 struct _GUPnPDIDLLiteWriterPrivate {
         GString *str;
 
-        SoupUri *url_base;
+        SoupURI *url_base;
 
         gboolean need_escape;
 };
@@ -135,14 +135,14 @@ append_escaped_text (GUPnPDIDLLiteWriter *writer,
         "<DIDL-Lite xmlns:" GUPNP_DIDL_LITE_WRITER_NAMESPACE_DC \
         "=\"http://purl.org/dc/elements/1.1/\" " \
         "xmlns:" GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP \
-        "=\"urn:schemas-upnp-org:metadata-1-0/upnp/\"" \
+        "=\"urn:schemas-upnp-org:metadata-1-0/upnp/\" " \
         "xmlns=\"urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/\""
 
 /**
  * gupnp_didl_lite_writer_start_didl_lite
  * @writer: A #GUPnPDIDLLiteWriter
  * @lang: The language the DIDL-Lite fragment is in, or NULL
- * @url_base: A #SoupUri specifying the URI relative to which incoming URI
+ * @url_base: A #SoupURI specifying the URI relative to which incoming URI
  * are resolved, or NULL
  * @need_escape: TRUE to force @writer to escape incoming string data
  *
@@ -151,7 +151,7 @@ append_escaped_text (GUPnPDIDLLiteWriter *writer,
 void
 gupnp_didl_lite_writer_start_didl_lite (GUPnPDIDLLiteWriter *writer,
                                         const char          *lang,
-                                        SoupUri             *url_base,
+                                        SoupURI             *url_base,
                                         gboolean             need_escape)
 {
         g_return_if_fail (GUPNP_IS_DIDL_LITE_WRITER (writer));
@@ -193,6 +193,7 @@ gupnp_didl_lite_writer_end_didl_lite (GUPnPDIDLLiteWriter *writer)
  * @writer: A #GUPnPDIDLLiteWriter
  * @id: The object ID
  * @parent_id: The parent object ID
+ * @child_count: The number of children or -1
  * @restricted: TRUE if this container is restricted
  * @searchable: TRUE if this container is searchable
  *
@@ -202,6 +203,7 @@ void
 gupnp_didl_lite_writer_start_container (GUPnPDIDLLiteWriter *writer,
                                         const char          *id,
                                         const char          *parent_id,
+                                        int                  child_count,
                                         gboolean             restricted,
                                         gboolean             searchable)
 {
@@ -224,6 +226,11 @@ gupnp_didl_lite_writer_start_container (GUPnPDIDLLiteWriter *writer,
         else
                 g_string_append (writer->priv->str, parent_id);
 
+        if (child_count >= 0) {
+                g_string_append_printf
+                        (writer->priv->str, "\" childCount=\"%d", child_count);
+        }
+
         g_string_append (writer->priv->str, "\" restricted=\"");
         g_string_append (writer->priv->str, restricted ? "true" : "false");
         g_string_append (writer->priv->str, "\" searchable=\"");
@@ -318,9 +325,14 @@ gupnp_didl_lite_resource_reset (GUPnPDIDLLiteResource *res)
 {
         res->uri           = NULL;
         res->import_uri    = NULL;
-        res->protocol_info = NULL;
+        res->network       = "*";
         res->protection    = NULL;
 
+        res->dlna_play_speed = GUPNP_DLNA_PLAY_SPEED_NORMAL;
+        res->dlna_conversion = GUPNP_DLNA_CONVERSION_NONE;
+        res->dlna_operation  = GUPNP_DLNA_OPERATION_NONE;
+        res->dlna_flags      = GUPNP_DLNA_FLAG_DLNA_V15;
+
         res->size             = -1;
         res->duration         = -1;
         res->bitrate          = -1;
@@ -344,23 +356,38 @@ void
 gupnp_didl_lite_writer_add_res (GUPnPDIDLLiteWriter   *writer,
                                 GUPnPDIDLLiteResource *res)
 {
-        SoupUri *uri;
+        SoupURI *uri;
         char *uri_str;
 
         g_return_if_fail (GUPNP_IS_DIDL_LITE_WRITER (writer));
         g_return_if_fail (writer->priv->str);
         g_return_if_fail (res != NULL);
         g_return_if_fail (res->uri != NULL);
-        g_return_if_fail (res->protocol_info != NULL);
+        g_return_if_fail (res->protocol != NULL);
+        g_return_if_fail (res->network != NULL);
+        g_return_if_fail (res->mime_type != NULL);
+        g_return_if_fail (res->dlna_profile != NULL);
 
         g_string_append (writer->priv->str, "<res protocolInfo=\"");
 
-        if (writer->priv->need_escape)
-                append_escaped_text (writer, res->protocol_info);
-        else
-                g_string_append (writer->priv->str, res->protocol_info);
+        g_string_append (writer->priv->str, res->protocol);
+        g_string_append_c (writer->priv->str, ':');
+        g_string_append (writer->priv->str, res->network);
+        g_string_append_c (writer->priv->str, ':');
+        g_string_append (writer->priv->str, res->mime_type);
 
-        g_string_append_c (writer->priv->str, '"');
+        g_string_append_printf (writer->priv->str,
+                                ":DLNA.ORG_PS=%d"
+                                ";DLNA.ORG_CI=%d"
+                                ";DLNA.ORG_OP=%.2x"
+                                ";DLNA.ORG_PN=%s"
+                                ";DLNA.ORG_FLAGS=%.8x%.24x\"",
+                                res->dlna_play_speed,
+                                res->dlna_conversion,
+                                res->dlna_operation,
+                                res->dlna_profile,
+                                res->dlna_flags,
+                                0);
 
         if (res->import_uri) {
                 g_string_append (writer->priv->str, " importUri=\"");
@@ -494,7 +521,7 @@ gupnp_didl_lite_writer_add_desc (GUPnPDIDLLiteWriter *writer,
                 g_string_append (writer->priv->str, " nameSpace=\"");
 
                 if (writer->priv->url_base || writer->priv->need_escape) {
-                        SoupUri *uri;
+                        SoupURI *uri;
                         char *uri_str;
 
                         uri = soup_uri_new_with_base (writer->priv->url_base,
@@ -544,7 +571,7 @@ begin_property (GUPnPDIDLLiteWriter *writer,
                 g_string_append (writer->priv->str, "=\"");
 
                 if (writer->priv->url_base || writer->priv->need_escape) {
-                        SoupUri *uri;
+                        SoupURI *uri;
                         char *uri_str;
 
                         uri = soup_uri_new_with_base (writer->priv->url_base,
